@@ -416,7 +416,7 @@ gst_avi_demux_handle_src_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
 
@@ -1052,6 +1052,7 @@ gst_avi_demux_parse_stream (GstElement * element, GstBuffer * buf)
   if (stream->pad)
     gst_object_unref (stream->pad);
   pad = stream->pad = gst_pad_new_from_template (templ, padname);
+  stream->last_flow = GST_FLOW_OK;
   g_free (padname);
 
   gst_pad_use_fixed_caps (pad);
@@ -2219,10 +2220,10 @@ swap_line (guint8 * d1, guint8 * d2, guint8 * tmp, gint bytes)
 static GstBuffer *
 gst_avi_demux_invert (avi_stream_context * stream, GstBuffer * buf)
 {
-  buf = gst_buffer_make_writable (buf);
   gint y, h = stream->strf.vids->height, w = stream->strf.vids->width;
   guint8 *tmp = NULL;
 
+  buf = gst_buffer_make_writable (buf);
   if (GST_BUFFER_SIZE (buf) < (w * h)) {
     GST_WARNING ("Buffer is smaller than reported Width x Height");
     return buf;
@@ -2240,6 +2241,22 @@ gst_avi_demux_invert (avi_stream_context * stream, GstBuffer * buf)
   return buf;
 }
 
+static gboolean
+gst_avi_demux_all_source_pads_unlinked (GstAviDemux * avi)
+{
+  gint i;
+
+  for (i = 0; i < avi->num_streams; ++i) {
+    if (gst_pad_is_linked (avi->stream[i].pad))
+      return FALSE;
+    /* ignore unlinked state if we haven't tried to push on this pad yet */
+    if (avi->stream[i].last_flow == GST_FLOW_OK)
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
 static GstFlowReturn
 gst_avi_demux_process_next_entry (GstAviDemux * avi)
 {
@@ -2294,9 +2311,13 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
             GST_TIME_FORMAT " on pad %s",
             GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
             GST_PAD_NAME (stream->pad));
-        if ((res = gst_pad_push (stream->pad, buf)) != GST_FLOW_OK &&
-            res != GST_FLOW_NOT_LINKED)
+        res = gst_pad_push (stream->pad, buf);
+        stream->last_flow = res;
+        if (res != GST_FLOW_OK && res != GST_FLOW_NOT_LINKED) {
+          GST_DEBUG_OBJECT (avi, "Flow on pad %s: %s",
+              GST_PAD_NAME (stream->pad), gst_flow_get_name (res));
           return res;
+        }
         processed = TRUE;
       }
     next:
@@ -2405,13 +2426,20 @@ gst_avi_demux_loop (GstPad * pad)
         gst_avi_demux_send_event (avi, avi->seek_event);
         avi->seek_event = NULL;
       }
-      if ((res = gst_avi_demux_stream_data (avi)) != GST_FLOW_OK)
+      if ((res = gst_avi_demux_stream_data (avi)) != GST_FLOW_OK) {
+        GST_DEBUG_OBJECT (avi, "stream_data flow: %s", gst_flow_get_name (res));
         goto pause;
+      }
       break;
     default:
       g_assert_not_reached ();
   }
 
+  if (gst_avi_demux_all_source_pads_unlinked (avi)) {
+    GST_DEBUG_OBJECT (avi, "all source pads unlinked, pausing");
+    goto pause;
+  }
+
   return;
 
 pause:
