@@ -18,37 +18,6 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-/**
- * SECTION:element-cutter
- *
- * Analyses the audio signal for periods of silence. The start and end of
- * silence is signalled by bus messages named
- * <classname>&quot;cutter&quot;</classname>.
- * The message's structure contains two fields:
- * <itemizedlist>
- * <listitem>
- *   <para>
- *   #GstClockTime
- *   <classname>&quot;timestamp&quot;</classname>:
- *   the timestamp of the buffer that triggered the message.
- *   </para>
- * </listitem>
- * <listitem>
- *   <para>
- *   gboolean
- *   <classname>&quot;above&quot;</classname>:
- *   %TRUE for begin of silence and %FALSE for end of silence.
- *   </para>
- * </listitem>
- * </itemizedlist>
- *
- * <refsect2>
- * <title>Example launch line</title>
- * |[
- * gst-launch -m filesrc location=foo.ogg ! decodebin ! audioconvert ! cutter ! autoaudiosink
- * ]| Show cut messages.
- * </refsect2>
- */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -58,13 +27,20 @@
 #include "gstcutter.h"
 #include "math.h"
 
-GST_DEBUG_CATEGORY_STATIC (cutter_debug);
+GST_DEBUG_CATEGORY (cutter_debug);
 #define GST_CAT_DEFAULT cutter_debug
 
 #define CUTTER_DEFAULT_THRESHOLD_LEVEL    0.1
 #define CUTTER_DEFAULT_THRESHOLD_LENGTH  (500 * GST_MSECOND)
 #define CUTTER_DEFAULT_PRE_LENGTH        (200 * GST_MSECOND)
 
+static GstElementDetails cutter_details = {
+  "Cutter",
+  "Filter/Editor/Audio",
+  "Audio Cutter to split audio into non-silent bits",
+  "Thomas <thomas@apestaart.org>",
+};
+
 static GstStaticPadTemplate cutter_src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -107,29 +83,28 @@ static void gst_cutter_get_property (GObject * object, guint prop_id,
 
 static GstFlowReturn gst_cutter_chain (GstPad * pad, GstBuffer * buffer);
 
-static gboolean gst_cutter_get_caps (GstPad * pad, GstCutter * filter);
+void gst_cutter_get_caps (GstPad * pad, GstCutter * filter);
 
 static void
 gst_cutter_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &cutter_src_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &cutter_sink_factory);
-  gst_element_class_set_details_simple (element_class, "Audio cutter",
-      "Filter/Editor/Audio",
-      "Audio Cutter to split audio into non-silent bits",
-      "Thomas Vander Stichele <thomas at apestaart dot org>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&cutter_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&cutter_sink_factory));
+  gst_element_class_set_details (element_class, &cutter_details);
 }
 
 static void
 gst_cutter_class_init (GstCutterClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   gobject_class->set_property = gst_cutter_set_property;
   gobject_class->get_property = gst_cutter_get_property;
@@ -137,25 +112,23 @@ gst_cutter_class_init (GstCutterClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_THRESHOLD,
       g_param_spec_double ("threshold", "Threshold",
           "Volume threshold before trigger",
-          -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          -G_MAXDOUBLE, G_MAXDOUBLE, 0.0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_THRESHOLD_DB,
       g_param_spec_double ("threshold-dB", "Threshold (dB)",
           "Volume threshold before trigger (in dB)",
-          -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          -G_MAXDOUBLE, G_MAXDOUBLE, 0.0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_RUN_LENGTH,
       g_param_spec_uint64 ("run-length", "Run length",
           "Length of drop below threshold before cut_stop (in nanoseconds)",
-          0, G_MAXUINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, G_MAXUINT64, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PRE_LENGTH,
       g_param_spec_uint64 ("pre-length", "Pre-recording buffer length",
           "Length of pre-recording buffer (in nanoseconds)",
-          0, G_MAXUINT64, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, G_MAXUINT64, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_LEAKY,
       g_param_spec_boolean ("leaky", "Leaky",
           "do we leak buffers when below threshold ?",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          FALSE, G_PARAM_READWRITE));
 
   GST_DEBUG_CATEGORY_INIT (cutter_debug, "cutter", 0, "Audio cutting");
 }
@@ -164,15 +137,16 @@ static void
 gst_cutter_init (GstCutter * filter, GstCutterClass * g_class)
 {
   filter->sinkpad =
-      gst_pad_new_from_static_template (&cutter_sink_factory, "sink");
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&cutter_sink_factory), "sink");
   filter->srcpad =
-      gst_pad_new_from_static_template (&cutter_src_factory, "src");
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&cutter_src_factory), "src");
 
   filter->threshold_level = CUTTER_DEFAULT_THRESHOLD_LEVEL;
   filter->threshold_length = CUTTER_DEFAULT_THRESHOLD_LENGTH;
   filter->silent_run_length = 0 * GST_SECOND;
   filter->silent = TRUE;
-  filter->silent_prev = FALSE;  /* previous value of silent */
 
   filter->pre_length = CUTTER_DEFAULT_PRE_LENGTH;
   filter->pre_run_length = 0 * GST_SECOND;
@@ -242,6 +216,7 @@ gst_cutter_chain (GstPad * pad, GstBuffer * buf)
   gdouble NCS = 0.0;            /* Normalized Cumulative Square of buffer */
   gdouble RMS = 0.0;            /* RMS of signal in buffer */
   gdouble NMS = 0.0;            /* Normalized Mean Square of buffer */
+  static gboolean silent_prev = FALSE;  /* previous value of silent */
   GstBuffer *prebuf;            /* pointer to a prebuffer element */
 
   g_return_val_if_fail (pad != NULL, GST_FLOW_ERROR);
@@ -252,11 +227,14 @@ gst_cutter_chain (GstPad * pad, GstBuffer * buf)
   g_return_val_if_fail (filter != NULL, GST_FLOW_ERROR);
   g_return_val_if_fail (GST_IS_CUTTER (filter), GST_FLOW_ERROR);
 
-  if (!filter->have_caps) {
-    if (!(gst_cutter_get_caps (pad, filter)))
-      return GST_FLOW_NOT_NEGOTIATED;
+  if (gst_audio_is_buffer_framed (pad, buf) == FALSE) {
+    g_warning ("audio buffer is not framed !\n");
+    return GST_FLOW_ERROR;
   }
 
+  if (!filter->have_caps)
+    gst_cutter_get_caps (pad, filter);
+
   in_data = (gint16 *) GST_BUFFER_DATA (buf);
   GST_LOG_OBJECT (filter, "length of prerec buffer: %" GST_TIME_FORMAT,
       GST_TIME_ARGS (filter->pre_run_length));
@@ -279,20 +257,17 @@ gst_cutter_chain (GstPad * pad, GstBuffer * buf)
       break;
   }
 
-  filter->silent_prev = filter->silent;
+  silent_prev = filter->silent;
 
   RMS = sqrt (NMS);
   /* if RMS below threshold, add buffer length to silent run length count
    * if not, reset
    */
-  GST_LOG_OBJECT (filter, "buffer stats: NMS %f, RMS %f, audio length %f", NMS,
-      RMS,
-      gst_guint64_to_gdouble (gst_audio_duration_from_pad_buffer
-          (filter->sinkpad, buf)));
+  GST_LOG_OBJECT (filter, "buffer stats: NMS %f, RMS %f, audio length %f",
+      NMS, RMS, gst_audio_duration_from_pad_buffer (filter->sinkpad, buf));
   if (RMS < filter->threshold_level)
     filter->silent_run_length +=
-        gst_guint64_to_gdouble (gst_audio_duration_from_pad_buffer
-        (filter->sinkpad, buf));
+        gst_audio_duration_from_pad_buffer (filter->sinkpad, buf);
   else {
     filter->silent_run_length = 0 * GST_SECOND;
     filter->silent = FALSE;
@@ -305,7 +280,7 @@ gst_cutter_chain (GstPad * pad, GstBuffer * buf)
   /* has the silent status changed ? if so, send right signal
    * and, if from silent -> not silent, flush pre_record buffer
    */
-  if (filter->silent != filter->silent_prev) {
+  if (filter->silent != silent_prev) {
     if (filter->silent) {
       GstMessage *m =
           gst_cutter_message_new (filter, FALSE, GST_BUFFER_TIMESTAMP (buf));
@@ -336,20 +311,16 @@ gst_cutter_chain (GstPad * pad, GstBuffer * buf)
   if (filter->silent) {
     filter->pre_buffer = g_list_append (filter->pre_buffer, buf);
     filter->pre_run_length +=
-        gst_guint64_to_gdouble (gst_audio_duration_from_pad_buffer
-        (filter->sinkpad, buf));
+        gst_audio_duration_from_pad_buffer (filter->sinkpad, buf);
     while (filter->pre_run_length > filter->pre_length) {
       prebuf = (g_list_first (filter->pre_buffer))->data;
       g_assert (GST_IS_BUFFER (prebuf));
       filter->pre_buffer = g_list_remove (filter->pre_buffer, prebuf);
       filter->pre_run_length -=
-          gst_guint64_to_gdouble (gst_audio_duration_from_pad_buffer
-          (filter->sinkpad, prebuf));
+          gst_audio_duration_from_pad_buffer (filter->sinkpad, prebuf);
       /* only pass buffers if we don't leak */
       if (!filter->leaky)
         gst_pad_push (filter->srcpad, prebuf);
-      else
-        gst_buffer_unref (prebuf);
     }
   } else
     gst_pad_push (filter->srcpad, buf);
@@ -357,28 +328,6 @@ gst_cutter_chain (GstPad * pad, GstBuffer * buf)
   return GST_FLOW_OK;
 }
 
-
-static gboolean
-gst_cutter_get_caps (GstPad * pad, GstCutter * filter)
-{
-  GstCaps *caps;
-  GstStructure *structure;
-
-  caps = gst_pad_get_caps (pad);
-  if (!caps) {
-    GST_INFO ("no caps on pad %s:%s", GST_DEBUG_PAD_NAME (pad));
-    return FALSE;
-  }
-  structure = gst_caps_get_structure (caps, 0);
-  gst_structure_get_int (structure, "width", &filter->width);
-  filter->max_sample = 1 << (filter->width - 1);        /* signed */
-  filter->have_caps = TRUE;
-
-  gst_caps_unref (caps);
-  return TRUE;
-}
-
-
 static void
 gst_cutter_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -404,12 +353,11 @@ gst_cutter_set_property (GObject * object, guint prop_id,
       break;
     case PROP_RUN_LENGTH:
       /* set the minimum length of the silent run required */
-      filter->threshold_length =
-          gst_guint64_to_gdouble (g_value_get_uint64 (value));
+      filter->threshold_length = g_value_get_uint64 (value);
       break;
     case PROP_PRE_LENGTH:
       /* set the length of the pre-record block */
-      filter->pre_length = gst_guint64_to_gdouble (g_value_get_uint64 (value));
+      filter->pre_length = g_value_get_uint64 (value);
       break;
     case PROP_LEAKY:
       /* set if the pre-record buffer is leaky or not */
@@ -466,3 +414,19 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     "cutter",
     "Audio Cutter to split audio into non-silent bits",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
+
+
+void
+gst_cutter_get_caps (GstPad * pad, GstCutter * filter)
+{
+  const GstCaps *caps = NULL;
+  GstStructure *structure;
+
+  caps = GST_PAD_CAPS (pad);
+  /* FIXME : Please change this to a better warning method ! */
+  g_assert (caps != NULL);
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "width", &filter->width);
+  filter->max_sample = 1 << (filter->width - 1);        /* signed */
+  filter->have_caps = TRUE;
+}
