@@ -1,6 +1,5 @@
 /* GStreamer Matroska muxer/demuxer
  * (c) 2003 Ronald Bultje <rbultje@ronald.bitfreak.net>
- * (c) 2011 Debarshi Ray <rishi@gnu.org>
  *
  * matroska-demux.h: matroska file/stream demuxer definition
  *
@@ -27,7 +26,6 @@
 
 #include "ebml-read.h"
 #include "matroska-ids.h"
-#include "matroska-read-common.h"
 
 G_BEGIN_DECLS
 
@@ -36,69 +34,76 @@ G_BEGIN_DECLS
 #define GST_MATROSKA_DEMUX(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_MATROSKA_DEMUX, GstMatroskaDemux))
 #define GST_MATROSKA_DEMUX_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_MATROSKA_DEMUX, GstMatroskaDemuxClass))
+  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_MATROSKA_DEMUX, GstMatroskaDemux))
 #define GST_IS_MATROSKA_DEMUX(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_MATROSKA_DEMUX))
-#define GST_IS_MATROSKA_DEMUX_CLASS(klass) \
+#define GST_IS_MATROSKA_DEMUX_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MATROSKA_DEMUX))
 
+#define GST_MATROSKA_DEMUX_MAX_STREAMS 64       
+
+typedef enum {
+  GST_MATROSKA_DEMUX_STATE_START,
+  GST_MATROSKA_DEMUX_STATE_HEADER,
+  GST_MATROSKA_DEMUX_STATE_DATA
+} GstMatroskaDemuxState;
+
+typedef struct _GstMatroskaDemuxIndex {
+  guint64        pos;   /* of the corresponding *cluster*! */
+  guint16        track; /* reference to 'num' */
+  guint64        time;  /* in nanoseconds */
+} GstMatroskaDemuxIndex;
+
 typedef struct _GstMatroskaDemux {
-  GstElement              parent;
-
-  /* < private > */
-
-  GstMatroskaReadCommon    common;
+  GstEbmlRead              parent;
 
   /* pads */
+  GstPad                  *sinkpad;
+  GstMatroskaTrackContext *src[GST_MATROSKA_DEMUX_MAX_STREAMS];
   GstClock                *clock;
+  guint                    num_streams;
   guint                    num_v_streams;
   guint                    num_a_streams;
   guint                    num_t_streams;
 
+  /* metadata */
+  gchar                   *muxing_app;
+  gchar                   *writing_app;
+  gint64                   created;
+
   /* state */
-  gboolean                 streaming;
+  GstMatroskaDemuxState    state;
   guint                    level_up;
-  guint64                  seek_block;
-  gboolean                 seek_first;
 
-  /* did we parse cues/tracks/segmentinfo already? */
-  gboolean                 tracks_parsed;
-  GList                   *seek_parsed;
+  /* did we parse metadata/cues already? */
+  gboolean                 metadata_parsed;
+  gboolean                 index_parsed;
 
-  /* cluster positions (optional) */
-  GArray                  *clusters;
+  /* start-of-segment */
+  guint64                  ebml_segment_start;
 
-  /* keeping track of playback position */
-  gboolean                 segment_running;
-  GstClockTime             last_stop_end;
-  GstClockTime             stream_start_time;
+  /* a cue (index) table */
+  GstMatroskaIndex        *index;
+  guint                    num_indexes;
 
-  GstEvent                *close_segment;
-  GstEvent                *new_segment;
+  /* timescale in the file */
+  guint64                  time_scale;
 
-  /* some state saving */
-  GstClockTime             cluster_time;
-  guint64                  cluster_offset;
-  guint64                  first_cluster_offset;
-  guint64                  next_cluster_offset;
+  /* length, position (time, ns) */
+  guint64                  pos;
+  guint64                  duration;
 
-  /* index stuff */
-  gboolean                 seekable;
-  gboolean                 building_index;
-  guint64                  index_offset;
-  GstEvent                *seek_event;
-  gboolean                 need_newsegment;
+  /* a possible pending seek */
+  gboolean                 seek_pending;
 
-  /* reverse playback */
-  GArray                  *seek_index;
-  gint                     seek_entry;
-
-  /* gap handling */
-  guint64                  max_gap_time;
+  gdouble                  segment_rate;
+  gint64                   segment_start;
+  gint64                   segment_stop;
+  gboolean                 segment_play;
 } GstMatroskaDemux;
 
 typedef struct _GstMatroskaDemuxClass {
-  GstElementClass parent;
+  GstEbmlReadClass parent;
 } GstMatroskaDemuxClass;
 
 gboolean gst_matroska_demux_plugin_init (GstPlugin *plugin);
