@@ -20,21 +20,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-osssrc
- *
- * This element lets you record sound using the Open Sound System (OSS).
- *
- * <refsect2>
- * <title>Example pipelines</title>
- * |[
- * gst-launch -v osssrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=mymusic.ogg
- * ]| will record sound from your sound card using OSS and encode it to an
- * Ogg/Vorbis file (this will only work if your mixer settings are right
- * and the right inputs enabled etc.)
- * </refsect2>
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -60,13 +45,16 @@
 #endif /* HAVE_OSS_INCLUDE_IN_SYS */
 
 #include "gstosssrc.h"
-#include "common.h"
-
-#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_EXTERN (oss_debug);
 #define GST_CAT_DEFAULT oss_debug
 
+static GstElementDetails gst_oss_src_details =
+GST_ELEMENT_DETAILS ("Audio Source (OSS)",
+    "Source/Audio",
+    "Capture from a sound card via OSS",
+    "Erik Walthinsen <omega@cse.ogi.edu>, " "Wim Taymans <wim@fluendo.com>");
+
 #define DEFAULT_DEVICE          "/dev/dsp"
 #define DEFAULT_DEVICE_NAME     ""
 
@@ -88,7 +76,6 @@ static void gst_oss_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
 static void gst_oss_src_dispose (GObject * object);
-static void gst_oss_src_finalize (GstOssSrc * osssrc);
 
 static GstCaps *gst_oss_src_getcaps (GstBaseSrc * bsrc);
 
@@ -131,30 +118,29 @@ gst_oss_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details_simple (element_class, "Audio Source (OSS)",
-      "Source/Audio",
-      "Capture from a sound card via OSS",
-      "Erik Walthinsen <omega@cse.ogi.edu>, " "Wim Taymans <wim@fluendo.com>");
+  gst_element_class_set_details (element_class, &gst_oss_src_details);
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&osssrc_src_factory));
 }
-
 static void
 gst_oss_src_class_init (GstOssSrcClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
+  GstBaseAudioSrcClass *gstbaseaudiosrc_class;
   GstAudioSrcClass *gstaudiosrc_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
+  gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
-  gobject_class->dispose = gst_oss_src_dispose;
-  gobject_class->finalize = (GObjectFinalizeFunc) gst_oss_src_finalize;
-  gobject_class->get_property = gst_oss_src_get_property;
-  gobject_class->set_property = gst_oss_src_set_property;
+  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_oss_src_dispose);
+  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_oss_src_get_property);
+  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_oss_src_set_property);
 
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_oss_src_getcaps);
 
@@ -220,27 +206,11 @@ gst_oss_src_get_property (GObject * object, guint prop_id,
 static void
 gst_oss_src_init (GstOssSrc * osssrc, GstOssSrcClass * g_class)
 {
-  const gchar *device;
-
   GST_DEBUG ("initializing osssrc");
 
-  device = g_getenv ("AUDIODEV");
-  if (device == NULL)
-    device = DEFAULT_DEVICE;
-
   osssrc->fd = -1;
-  osssrc->device = g_strdup (device);
+  osssrc->device = g_strdup (DEFAULT_DEVICE);
   osssrc->device_name = g_strdup (DEFAULT_DEVICE_NAME);
-  osssrc->probed_caps = NULL;
-}
-
-static void
-gst_oss_src_finalize (GstOssSrc * osssrc)
-{
-  g_free (osssrc->device);
-  g_free (osssrc->device_name);
-
-  G_OBJECT_CLASS (parent_class)->finalize ((GObject *) (osssrc));
 }
 
 static GstCaps *
@@ -252,23 +222,12 @@ gst_oss_src_getcaps (GstBaseSrc * bsrc)
   osssrc = GST_OSS_SRC (bsrc);
 
   if (osssrc->fd == -1) {
-    GST_DEBUG_OBJECT (osssrc, "device not open, using template caps");
-    return NULL;                /* base class will get template caps for us */
+    caps = gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD
+            (bsrc)));
+  } else {
+    caps = gst_oss_helper_probe_caps (osssrc->fd);
   }
 
-  if (osssrc->probed_caps) {
-    GST_LOG_OBJECT (osssrc, "Returning cached caps");
-    return gst_caps_ref (osssrc->probed_caps);
-  }
-
-  caps = gst_oss_helper_probe_caps (osssrc->fd);
-
-  if (caps) {
-    osssrc->probed_caps = gst_caps_ref (caps);
-  }
-
-  GST_INFO_OBJECT (osssrc, "returning caps %" GST_PTR_FORMAT, caps);
-
   return caps;
 }
 
@@ -289,6 +248,31 @@ ilog2 (gint x)
   return (x & 0x0000003f) - 1;
 }
 
+#define SET_PARAM(_oss, _name, _val)            \
+G_STMT_START {                                  \
+  int _tmp = _val;                              \
+  if (ioctl(_oss->fd, _name, &_tmp) == -1) {    \
+    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ, \
+        ("Unable to set param "G_STRINGIFY (_name)": %s",        \
+                g_strerror (errno)),            \
+        (NULL));                                \
+    return FALSE;                               \
+  }                                             \
+  GST_DEBUG_OBJECT (_oss, G_STRINGIFY (_name)" %d", _tmp);      \
+} G_STMT_END
+
+#define GET_PARAM(_oss, _name, _val)    \
+G_STMT_START {                                  \
+  if (ioctl(oss->fd, _name, _val) == -1) {      \
+    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ, \
+        ("Unable to get param "G_STRINGIFY (_name)": %s",        \
+                g_strerror (errno)),            \
+        (NULL));                                \
+    return FALSE;                               \
+  }                                             \
+  GST_DEBUG_OBJECT (_oss, G_STRINGIFY (_name)" %d", _val);      \
+} G_STMT_END
+
 static gint
 gst_oss_src_get_format (GstBufferFormat fmt)
 {
@@ -344,14 +328,8 @@ gst_oss_src_open (GstAudioSrc * asrc)
   mode |= O_NONBLOCK;
 
   oss->fd = open (oss->device, mode, 0);
-  if (oss->fd == -1) {
-    switch (errno) {
-      case EACCES:
-        goto no_permission;
-      default:
-        goto open_failed;
-    }
-  }
+  if (oss->fd == -1)
+    goto open_failed;
 
   if (!oss->mixer) {
     oss->mixer = gst_ossmixer_new ("/dev/mixer", GST_OSS_MIXER_CAPTURE);
@@ -363,20 +341,11 @@ gst_oss_src_open (GstAudioSrc * asrc)
   }
   return TRUE;
 
-no_permission:
-  {
-    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
-        (_("Could not open audio device for recording. "
-                "You don't have permission to open the device.")),
-        GST_ERROR_SYSTEM);
-    return FALSE;
-  }
 open_failed:
   {
     GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
-        (_("Could not open audio device for recording.")),
         ("Unable to open device %s for recording: %s",
-            oss->device, g_strerror (errno)));
+            oss->device, g_strerror (errno)), (NULL));
     return FALSE;
   }
 }
@@ -395,8 +364,6 @@ gst_oss_src_close (GstAudioSrc * asrc)
     oss->mixer = NULL;
   }
 
-  gst_caps_replace (&oss->probed_caps, NULL);
-
   return TRUE;
 }
 
@@ -421,20 +388,20 @@ gst_oss_src_prepare (GstAudioSrc * asrc, GstRingBufferSpec * spec)
 
   tmp = ilog2 (spec->segsize);
   tmp = ((spec->segtotal & 0x7fff) << 16) | tmp;
-  GST_DEBUG_OBJECT (oss, "set segsize: %d, segtotal: %d, value: %08x",
-      spec->segsize, spec->segtotal, tmp);
+  GST_DEBUG ("set segsize: %d, segtotal: %d, value: %08x", spec->segsize,
+      spec->segtotal, tmp);
 
-  SET_PARAM (oss, SNDCTL_DSP_SETFRAGMENT, tmp, "SETFRAGMENT");
+  SET_PARAM (oss, SNDCTL_DSP_SETFRAGMENT, tmp);
 
-  SET_PARAM (oss, SNDCTL_DSP_RESET, 0, "RESET");
+  SET_PARAM (oss, SNDCTL_DSP_RESET, 0);
 
-  SET_PARAM (oss, SNDCTL_DSP_SETFMT, fmt, "SETFMT");
+  SET_PARAM (oss, SNDCTL_DSP_SETFMT, fmt);
   if (spec->channels == 2)
-    SET_PARAM (oss, SNDCTL_DSP_STEREO, 1, "STEREO");
-  SET_PARAM (oss, SNDCTL_DSP_CHANNELS, spec->channels, "CHANNELS");
-  SET_PARAM (oss, SNDCTL_DSP_SPEED, spec->rate, "SPEED");
+    SET_PARAM (oss, SNDCTL_DSP_STEREO, 1);
+  SET_PARAM (oss, SNDCTL_DSP_CHANNELS, spec->channels);
+  SET_PARAM (oss, SNDCTL_DSP_SPEED, spec->rate);
 
-  GET_PARAM (oss, SNDCTL_DSP_GETISPACE, &info, "GETISPACE");
+  GET_PARAM (oss, SNDCTL_DSP_GETISPACE, &info);
 
   spec->segsize = info.fragsize;
   spec->segtotal = info.fragstotal;
@@ -446,8 +413,8 @@ gst_oss_src_prepare (GstAudioSrc * asrc, GstRingBufferSpec * spec)
   oss->bytes_per_sample = (spec->width / 8) * spec->channels;
   memset (spec->silence_sample, 0, spec->bytes_per_sample);
 
-  GST_DEBUG_OBJECT (oss, "got segsize: %d, segtotal: %d, value: %08x",
-      spec->segsize, spec->segtotal, tmp);
+  GST_DEBUG ("got segsize: %d, segtotal: %d, value: %08x", spec->segsize,
+      spec->segtotal, tmp);
 
   return TRUE;
 
@@ -487,12 +454,12 @@ gst_oss_src_unprepare (GstAudioSrc * asrc)
 
 couldnt_close:
   {
-    GST_DEBUG_OBJECT (asrc, "Could not close the audio device");
+    GST_DEBUG ("Could not close the audio device");
     return FALSE;
   }
 couldnt_reopen:
   {
-    GST_DEBUG_OBJECT (asrc, "Could not reopen the audio device");
+    GST_DEBUG ("Could not reopen the audio device");
     return FALSE;
   }
 }
@@ -530,7 +497,12 @@ gst_oss_src_delay (GstAudioSrc * asrc)
 static void
 gst_oss_src_reset (GstAudioSrc * asrc)
 {
-  /* There's nothing we can do here really: OSS can't handle access to the
-   * same device/fd from multiple threads and might deadlock or blow up in
-   * other ways if we try an ioctl SNDCTL_DSP_RESET or similar */
+  GstOssSrc *oss;
+
+  //gint ret;
+
+  oss = GST_OSS_SRC (asrc);
+
+  /* deadlocks on my machine... */
+  //ret = ioctl (oss->fd, SNDCTL_DSP_RESET, 0);
 }
